#  Script to generate the cell coordinates
#  for which we are going to calculate
#  the neighbor list.

# Inports:
import numpy as np
import itertools as it

# Inputs:
size = [2,2,2]       # Number of times we are replicating the primitive cell
center = [0,0,0]     # The position (x,y,z) in space where the ceneter is
primitive_cell = [[4,0,0],[0,4,0],[0,0,4]] # The primitive cell

cell_coords = np.array(list((it.product(np.arange(size[0]),
                                        np.arange(size[1]),
                                        np.arange(size[2])))))
cell_coords_centered = cell_coords + center
cell_coords_centered = np.dot(cell_coords_centered, primitive_cell)

# return the cells coordinates generated
print(cell_coords_centered)



import unittest
import numpy as np
from debyetools.ndeb import nDeb
from debyetools.aux_functions import gen_Ts
import debyetools.potentials as potentials

class CpTestCase(unittest.TestCase):
    def setUp(self):
        #self.NL = PairAnalysisCalculator()
        pass

    def test_Cp_Al_fcc(self):
        """ Test evaluation of Cp for Al fcc at a given T and V using the 3rd order Birch-Murnaghan EOS."""

        nu, m = 0.32, 0.026981500000000002
        Tmelting = 933
        p_EOS = [-3.617047894e+05, 9.929931142e-06, 7.618619745e+10, 4.591924487e+00]
        p_intanh = 0, 1
        p_electronic = [3.8027342892e-01, -1.8875015171e-02, 5.3071034596e-04, -7.0100707467e-06]
        p_defects = 8.46, 1.69, Tmelting, 0.1
        p_anh = 0, 0, 0

        EOS_name = 'BM'
        EOS_BM = getattr(potentials, EOS_name)()
        EOS_BM.pEOS = p_EOS
        EOS_BM.V0 = p_EOS[1]
        ndeb_BM = nDeb(nu, m, p_intanh, EOS_BM, p_electronic, p_defects, p_anh, EOS_name)

        Tx, Vx = 9.3300000000000E+02, 1.07790131286e-05

        np.testing.assert_almost_equal(ndeb_BM.eval_props(Tx,Vx)['Cp'], 3.32496610718e+01, decimal=3)

if __name__=='__main__':
    unittest.main()



import unittest
from debyetools.ndeb import nDeb
from debyetools.aux_functions import gen_Ts
import numpy as np
import debyetools.potentials as potentials

class FminTestCase(unittest.TestCase):
    def setUp(self):
        self.nu, self.m = 0.31681, 0.026981500000000002
        self.Tmelting = 933
        self.p_EOS = [-3.617047894e+05, 9.929931142e-06, 7.618619745e+10, 4.591924487e+00]
        self.p_intanh = 0, 1
        self.p_electronic = [3.8027342892e-01, -1.8875015171e-02, 5.3071034596e-04, -7.0100707467e-06]
        self.p_defects = 8.46, 1.69, self.Tmelting, 0.1
        self.p_anh = 0,0,0
        self.T_initial, self.T_final = 0.1, 1000.1
        self.number_Temps = 51

    def test_Free_energy_minimization_Al_fcc_BM(self):
        """ Test V(T) calculation by free energy minimization. BM."""

        EOS_name = 'BM'
        EOS_BM = getattr(potentials, EOS_name)()
        EOS_BM.pEOS = self.p_EOS
        EOS_BM.V0 = self.p_EOS[1]
        ndeb_BM = nDeb(self.nu, self.m, self.p_intanh, EOS_BM, self.p_electronic, self.p_defects, self.p_anh, EOS_name)

        T = gen_Ts(self.T_initial, self.T_final, self.number_Temps)
        T, V = ndeb_BM.min_F(T,self.p_EOS[1])

        self.assertAlmostEqual(35.166117790049896, ndeb_BM.eval_props(T[-1],V[-1])['Cp'])

    def test_Free_energy_minimization_Al_fcc_RV(self):
        """ Test V(T) calculation by free energy minimization. RV."""

        EOS_name = 'RV'
        EOS_BM = getattr(potentials, EOS_name)()
        EOS_BM.pEOS = self.p_EOS
        EOS_BM.V0 = self.p_EOS[1]
        ndeb_BM = nDeb(self.nu, self.m, self.p_intanh, EOS_BM, self.p_electronic, self.p_defects, self.p_anh, EOS_name)

        T = gen_Ts(self.T_initial, self.T_final, self.number_Temps)
        T, V = ndeb_BM.min_F(T,self.p_EOS[1])

        self.assertAlmostEqual(37.96026289773106, ndeb_BM.eval_props(T[-1],V[-1])['Cp'])

    def test_Free_energy_minimization_Al_fcc_Morse(self):
        """ Test V(T) calculation by free energy minimization. Morse."""

        EOS_name = 'MP'
        p_EOS = np.array([3.492281316e-01, 9.977375168e-01, 3.246481751e+00])

        formula = 'AlAlAlAl'
        a = 4.0396918604
        primitive_cell = np.array([[a, 0, 0], [0, a, 0], [0, 0, a]])
        basis_vectors = np.array([[0,0,0],[0, .5,.5],[.5,0,.5],[.5,.5,0]])
        cutoff = 5.
        number_of_neighbor_levels = 6
        EOS_BM = getattr(potentials, EOS_name)(formula, primitive_cell, basis_vectors, cutoff, number_of_neighbor_levels)
        EOS_BM.pEOS = p_EOS
        EOS_BM.V0 = p_EOS[1]
        ndeb_Morse = nDeb(self.nu, self.m, self.p_intanh, EOS_BM,
                        self.p_electronic, self.p_defects, self.p_anh, EOS_name, formula, primitive_cell, basis_vectors, cutoff, number_of_neighbor_levels)

        T = gen_Ts(self.T_initial, self.T_final, self.number_Temps)
        T, V = ndeb_Morse.min_F(T, self.p_EOS[1])

        self.assertAlmostEqual(32.15669813382092, ndeb_Morse.eval_props(T[-1],V[-1])['Cp'])

    def test_Free_energy_minimization_Al_fcc_EAM(self):
        """ Test V(T) calculation by free energy minimization. EAM."""

        EOS_name = 'EAM'
        p_EOS = np.array([1.864283e-02, 1.087716e+00, 1.267761e+00, 1.016616e+00, 1.005353e+00, 2.981997e+00, 2.563705e-07, 1.105120e+00,1.815219e+00, 1.493627e+00])
        formula = 'AlAlAlAl'
        a = 4.0396918604
        primitive_cell = np.array([[a, 0, 0], [0, a, 0], [0, 0, a]])
        basis_vectors = np.array([[0,0,0],[0, .5,.5],[.5,0,.5],[.5,.5,0]])
        cutoff = 5.
        number_of_neighbor_levels = 6
        EOS_BM = getattr(potentials, EOS_name)(formula, primitive_cell, basis_vectors, cutoff, number_of_neighbor_levels, parameters=p_EOS)
        EOS_BM.V0 = 10E-6
        ndeb_Morse = nDeb(self.nu, self.m, self.p_intanh, EOS_BM,
                          self.p_electronic, self.p_defects, self.p_anh, EOS_name, formula, primitive_cell, basis_vectors, cutoff, number_of_neighbor_levels)

        T = gen_Ts(self.T_initial, self.T_final, self.number_Temps)
        T, V = ndeb_Morse.min_F(T,self.p_EOS[1])

        self.assertAlmostEqual(36.127494489465, ndeb_Morse.eval_props(T[-1], V[-1])['Cp'])


if __name__=='__main__':
    unittest.main()



import itertools as it
import re
import numpy as np
hola
def c_types(atom_types):
    """
    returns all the pair types combinations.

    :param list_of_str atom_types: the types of each atom in the primitive cell in the same order as the basis vectors.
    """
    types_all = re.findall('[A-Z][^A-Z]*', atom_types)
    ptypes=list(set([s for s in types_all]))
    ptypes.sort()
    combs_types = list(it.combinations_with_replacement(ptypes, r=2))
    combs_types = [A[0]+'-'+A[1] for A in combs_types]
    return combs_types, types_all

def generate_cells_coordinates(size, primitive_cell,center):
    """ generate the cell coordinates for which we are going
    to calculate the neighbor list.

    :param array size: Number of times we are replicating the primitive cell
    :param array primitive_cell: The primitive cell
    :param array center: The position in space where the system of reference is
    """

    cell_coords = np.array(list((it.product(np.arange(size[0]),
                                            np.arange(size[1]),
                                            np.arange(size[2])))))
    cell_coords_centered = cell_coords + center
    cell_coords_centered = np.dot(cell_coords_centered, primitive_cell)

    return cell_coords_centered



import unittest
import numpy as np
from debyetools.ndeb import nDeb
from debyetools.aux_functions import gen_Ts
from debyetools.fs_compound_db import fit_FS
import debyetools.potentials as potentials
from debyetools.electronic import fit_electronic
from debyetools.poisson import poisson_ratio
from debyetools.aux_functions import load_doscar, load_V_E, load_EM, load_cell

class CpTestCase(unittest.TestCase):
    def setUp(self):
        #self.NL = PairAnalysisCalculator()
        pass

    def test_Complete_Al_fcc_BM4(self):
        """ Test complete algorithm to calculate TP for Al fcc using the 4th order Birch-Murnaghan EOS."""

        folder_name = './tests/inpt_files/Al_fcc'
        # EOS parametrization
        #=========================
        V_DFT, E_DFT = load_V_E(folder_name, folder_name+'/CONTCAR.5', units='J/mol')
        EOS_name = 'BM4'
        initial_parameters =  [-3.6e+05, 9.9e-06, -7.8e+10, 4.7e+00, 1.e-10]
        eos_BM4 = getattr(potentials,EOS_name)()
        eos_BM4.fitEOS(V_DFT, E_DFT, initial_parameters=initial_parameters)
        p_EOS = eos_BM4.pEOS
        #=========================

        # Electronic Contributions
        #=========================
        p_el_inittial = [3.8027342892e-01, -1.8875015171e-02,
                         5.3071034596e-04, -7.0100707467e-06]
        E, N, Ef = load_doscar(folder_name+'/DOSCAR.EvV.')
        p_electronic = fit_electronic(V_DFT, p_el_inittial,E,N,Ef)
        #=========================

        # Other Contributions parametrization
        #=========================
        Tmelting = 933
        p_defects = 8.46, 1.69, Tmelting, 0.1
        p_intanh = 0, 1
        p_anh = 0, 0, 0
        #=========================

        # Poisson's ratio
        #=========================
        EM = EM = load_EM(folder_name+'/OUTCAR.eps')
        nu = poisson_ratio(EM)
        #=========================

        # F minimization
        #=========================
        m = 0.026981500000000002
        ndeb_BM4 = nDeb(nu, m, p_intanh, eos_BM4, p_electronic,
                        p_defects, p_anh)

        T_initial, T_final, number_Temps = 0.1, 1000, 10
        T = gen_Ts(T_initial, T_final, number_Temps)

        T, V = ndeb_BM4.min_F(T,p_EOS[1])
        #=========================

        # Evaluations
        #=========================
        tprops_dict = ndeb_BM4.eval_props(T,V)

        T_from = 298.15
        T_to = 1000
        #=========================

        # FS comp db parameters
        #=========================
        FS_db_params = fit_FS(tprops_dict,T_from, T_to)
        #=========================

        np.testing.assert_array_almost_equal(FS_db_params['Cp'],
                                             [2.85699998e+02,-2.42049765e-01,2.73093597e+06,1.19257103e-04,-3.98802525e+03,1.00000000e+00], decimal=2)

    def test_Complete_Al_fcc_Morse(self):
        """ Test complete algorithm to calculate TP for Al fcc using the 4th order Birch-Murnaghan EOS."""

        folder_name = './tests/inpt_files/Al_fcc'
        # EOS parametrization
        #=========================
        V_DFT, E_DFT = load_V_E(folder_name, folder_name + '/CONTCAR.5', units='J/mol')

        formula, primitive_cell, sbasis_vectors = load_cell(folder_name+'/CONTCAR.5')

        EOS_name = 'MP'
        cutoff = 5
        number_of_neighbor_levels = 3
        eos_Morse = getattr(potentials,EOS_name)(formula, primitive_cell, sbasis_vectors, cutoff, number_of_neighbor_levels, units='J/mol')
        initial_parameters = np.array([0.35, 1, 3.5])

        eos_Morse.fitEOS(V_DFT, E_DFT, initial_parameters=initial_parameters)
        p_EOS = eos_Morse.pEOS
        #=========================

        # Electronic Contributions
        #=========================
        p_el_inittial = [3.8027342892e-01, -1.8875015171e-02,
                         5.3071034596e-04, -7.0100707467e-06]
        E, N, Ef = load_doscar(folder_name+'/DOSCAR.EvV.')
        p_electronic = fit_electronic(V_DFT, p_el_inittial,E,N,Ef)
        #=========================

        # Other Contributions parametrization
        #=========================
        Tmelting = 933
        p_defects = 8.46, 1.69, Tmelting, 0.1
        p_intanh = 0, 1
        p_anh = 0, 0, 0
        #=========================

        # Poisson's ratio
        #=========================
        EM = EM = load_EM(folder_name+'/OUTCAR.eps')
        nu = poisson_ratio(EM)
        #=========================

        # F minimization
        #=========================
        m = 0.026981500000000002
        ndeb_Morse = nDeb(nu, m, p_intanh, eos_Morse, p_electronic,
                          p_defects, p_anh)

        T_initial, T_final, number_Temps = 0.1, 1000, 10
        T = gen_Ts(T_initial, T_final, number_Temps)

        T, V = ndeb_Morse.min_F(T,ndeb_Morse.EOS.V0)
        #=========================

        # Evaluations
        #=========================
        tprops_dict = ndeb_Morse.eval_props(T, V)
        #=========================

        # FS comp db parameters
        #=========================
        T_from = 298.15
        T_to = 1000
        FS_db_params = fit_FS(tprops_dict,T_from, T_to)
        #=========================

        print(FS_db_params['Cp'])
        np.testing.assert_array_almost_equal(FS_db_params['Cp'],
                                             [1.11898466e+02,-8.11995443e-02,7.22119591e+05,4.29282477e-05,
                                              -1.31482568e+03,1.00000000e+00], decimal=2)

if __name__=='__main__':
    unittest.main()



import unittest
import numpy as np
from debyetools.potentials import MP, BM, EAM
from debyetools.aux_functions import load_V_E, load_cell
class EOSparametrizationTestCase(unittest.TestCase):
    def setUp(self):
        self.V_DFT = np.array([7.2328381349E-06,7.4766214899E-06,7.7258220323E-06,7.9804992917E-06,8.2407127976E-06,8.5065220794E-06,8.7779866668E-06,9.0551660893E-06,9.3381198763E-06,9.6269075575E-06,9.9215886624E-06,1.0222222720E-05,1.0528869261E-05,1.0841587814E-05,1.1160437909E-05,1.1485479075E-05,1.1816770842E-05,1.2154372740E-05,1.2498344297E-05,1.2848745044E-05,1.3205634510E-05])
        self.E_DFT = np.array([-3.1142624910E+05,-3.2250917927E+05,-3.3187138662E+05,-3.3970381602E+05,-3.4613131971E+05,-3.5129770939E+05,-3.5530574212E+05,-3.5828253759E+05,-3.6027737577E+05,-3.6141349280E+05,-3.6177140584E+05,-3.6144576720E+05,-3.6049509535E+05,-3.5899479371E+05,-3.5699510714E+05,-3.5454316879E+05,-3.5173001277E+05,-3.4858366812E+05,-3.4518407309E+05,-3.4152425661E+05,-3.3764778189E+05])

        self.primitive_cell =lambda a: np.array([[a, 0, 0], [0, a, 0], [0, 0, a]])
        self.basis_vectors = np.array([[0,0,0],[0, .5,.5],[.5,0,.5],[.5,.5,0]])

        self.formula = 'AlAlAlAl'
        self.cutoff = 5.
        self.number_of_neighbor_levels = 3

    def test_EOS_Morse_Al_fcc_evaluations(self):
        """ Test evaluation of Morse potential using Al fcc DFT data."""

        p_EOS = np.array([3.492281316e-01, 9.977375168e-01, 3.246481751e+00])
        eos_Morse = MP(self.formula, self.primitive_cell(4.0396918604), self.basis_vectors, self.cutoff, self.number_of_neighbor_levels, units='J/mol', parameters = p_EOS)

        E_model = [eos_Morse.E0(Vi) for Vi in self.V_DFT]

        self.assertTrue(np.sqrt(np.sum(np.array([(Ei-Emi)**2 for Ei, Emi in zip(self.E_DFT, E_model)])))/np.abs(np.mean(self.E_DFT))*100<0.5)

    def test_EOS_Morse_Al_fcc_evaluations_eV_units(self):
        """ Test evaluation of Morse potential using Al fcc DFT data, eV/atom units."""
        V_DFT = self.V_DFT/(1e-30*6.02e23)
        E_DFT = self.E_DFT/(0.160218e-18*6.02214e23)

        p_EOS = np.array([3.492281316e-01, 9.977375168e-01, 3.246481751e+00])
        eos_Morse = MP(self.formula, self.primitive_cell(4.0396918604), self.basis_vectors, self.cutoff, self.number_of_neighbor_levels, units='eV/atom', parameters = p_EOS)

        E_model = [eos_Morse.E0(Vi) for Vi in V_DFT]

        self.assertTrue(np.sqrt(np.sum(np.array([(Ei-Emi)**2 for Ei, Emi in zip(E_DFT, E_model)])))/np.abs(np.mean(E_DFT))*100<0.5)

    def test_EOS_Morse_Al_fcc_fitting(self):
        """ Test fitting of Morse potential using Al fcc DFT data."""
        eos_Morse = MP(self.formula, self.primitive_cell(4.0396918604), self.basis_vectors, self.cutoff, self.number_of_neighbor_levels, units='J/mol')
        initial_parameters = np.array([0.35, 1, 3.5])

        eos_Morse.fitEOS(self.V_DFT, self.E_DFT, initial_parameters=initial_parameters)

        np.testing.assert_array_almost_equal(eos_Morse.pEOS, np.array([3.492281316e-01, 9.977375168e-01, 3.246481751e+00]))

    def test_EOS_BM3_Al_fcc_eval(self):
        """ Test fitting of BM3 potential using Al fcc DFT data."""
        pEOS = np.array([-3.617047894e+05, 9.929931142e-06, 7.618619745e+10, 4.591924487e+00])
        eos_BM3 = BM(parameters=pEOS)

        E_model = [eos_BM3.E0(Vi) for Vi in self.V_DFT]

        self.assertTrue(np.sqrt(np.sum(np.array([(Ei-Emi)**2 for Ei, Emi in zip(self.E_DFT, E_model)])))/np.abs(np.mean(self.E_DFT))*100<0.5)

    def test_EOS_BM3_Al_fcc_fit(self):
        """ Test fitting of BM3 potential using Al fcc DFT data."""
        eos_BM3 = BM()
        initial_parameters = np.array([-3.617047894e+05, 9.929931142e-06, 7.618619745e+10, 4.591924487e+00])
        pEOS = eos_BM3.fitEOS(self.V_DFT, self.E_DFT,initial_parameters=initial_parameters)

        E_model = [eos_BM3.E0(Vi) for Vi in self.V_DFT]

        self.assertTrue(np.sqrt(np.sum(np.array([(Ei-Emi)**2 for Ei, Emi in zip(self.E_DFT, E_model)])))/np.abs(np.mean(self.E_DFT))*100<0.5)

    # def test_EOS_EAM_Al_fcc_fitting(self):
    #     """ Test fitting of EAM potential using Al fcc DFT data."""
    #     eos_Morse = EAM(self.formula, self.primitive_cell(4.0396918604), self.basis_vectors, self.cutoff,
    #                     self.number_of_neighbor_levels, units='J/mol')
    #     initial_parameters = np.array([3.601752881e-01, 1.090829630e+00, 1.098542954e+00, 1.012121606e+00,
    #                                    1.004830571e+00, 1.519642057e+00, 1.883696186e-01, 1.103658726e+00,
    #                                    9.722673674e-01, 1.502896332e+00])
    #
    #     eos_Morse.fitEOS(self.V_DFT, self.E_DFT, initial_parameters=initial_parameters)
    #
    #     np.testing.assert_array_almost_equal(eos_Morse.pEOS, np.array([1.86428338e-02,1.08771586e+00,1.26776095e+00,1.01661602e+00,1.00535350e+00,2.98199738e+00,2.56370530e-07,1.10512008e+00,1.81521927e+00,1.49362736e+00]),decimal=3)

    def test_EOS_BM3_Al_fcc_fit_read_inpts(self):
        """ Test fitting of BM3 potential using Al fcc DFT data."""

        folder_name = './tests/inpt_files/Al_fcc'#../tests/inpt_files/Al_fcc'
        V_DFT, E_DFT = load_V_E(folder_name, folder_name+'/CONTCAR.5', units='J/mol')

        eos_BM3 = BM()
        initial_parameters = np.array([-3.617047894e+05, 9.929931142e-06, 7.618619745e+10, 4.591924487e+00])
        pEOS = eos_BM3.fitEOS(V_DFT, E_DFT,initial_parameters=initial_parameters)

        E_model = [eos_BM3.E0(Vi) for Vi in self.V_DFT]

        self.assertTrue(np.sqrt(np.sum(np.array([(Ei-Emi)**2 for Ei, Emi in zip(self.E_DFT, E_model)])))/np.abs(np.mean(self.E_DFT))*100<0.5)

    def test_EOS_Morse_Al_fcc_fitting_reading_from_file(self):
        """ Test fitting of Morse potential using Al fcc DFT data."""
        folder_name = './tests/inpt_files/Al_fcc'
        V_DFT, E_DFT = load_V_E(folder_name, folder_name + '/CONTCAR.5', units='J/mol')

        formula, primitive_cell, sbasis_vectors = load_cell(folder_name+'/CONTCAR.5')

        eos_Morse = MP(formula, primitive_cell, sbasis_vectors, self.cutoff, self.number_of_neighbor_levels, units='J/mol')
        initial_parameters = np.array([0.35, 1, 3.5])

        eos_Morse.fitEOS(V_DFT, E_DFT, initial_parameters=initial_parameters)

        np.testing.assert_array_almost_equal(eos_Morse.pEOS, np.array([3.492281316e-01, 9.977375168e-01, 3.246481751e+00]))

if __name__=='__main__':
    unittest.main()



import numpy as np

class Anharmonicity:
    """
    Instance for the excess contribution to the free energy.

    :param float s0,s1,s2: Parameters of the A(V) term.
    """
    def __init__(self,s0,s1,s2):
        self.s0=s0
        self.s1=s1
        self.s2=s2
    def A(self,V):
        return self.s0 + self.s1*V + self.s2*V**2
    def dAdV_T(self,V):
        return 2*V*self.s2+self.s1
    def d2AdV2_T(self,V):
        return 2*self.s2
    def d3AdV3_T(self,V):
        return 0
    def d4AdV4_T(self,V):
        return 0

    def E(self,T,V):
        """
        Internal energy due the excess term.

        :param float T: Temperature.
        :param float V: Volume.
        """
        return 1/2*self.A(V)*T**2
    def S(self,T,V):
        """
        Entropy due the excess term.

        :param float T: Temperature.
        :param float V: Volume.
        """
        return self.A(V)*T
    def F(self,T,V):
        """
        Free energy due the excess term.

        :param float T: Temperature.
        :param float V: Volume.
        """
        return -1/2*self.A(V)*T**2
    def d2FdT2_V(self,T,V):
        return -self.A(V)
    def d2FdV2_T(self,T,V):
        return -self.d2AdV2_T(V)*T**2/2
    def d3FdV3_T(self,T,V):
        return -self.d3AdV3_T(V)*T**2/2
    def d4FdV4_T(self,T,V):
        return -self.d4AdV4_T(V)*T**2/2
    def d2FdVdT(self,T,V):
        return -self.dAdV_T(V)*T
    def d3FdV2dT(self,T,V):
        return -self.d2AdV2_T(V)*T
    def d3FdVdT2(self,T,V):
        return -self.dAdV_T(V)

class intAnharmonicity:
    def __init__(self,a0=0,m0=1,V0=1):
        """
        Instantiate the corrections required to consider temperature dependence on the Debye temperature.

        :param float a0: Parameter of the intrinsic anharmonicity correction function.
        :param float m0: Pseudo-Gruneisen parameter of the intrinsic anharmonicity correction function.
        :param float V0: Equilibrium volume.
        """
        self.a0 = a0
        self.m0 = m0
        self.V0 = V0
    def an(self,V):
        """
        Intrinsic anharmonicity parameter.

        :param float V: Volume
        """
        return self.a0*(V/self.V0)**self.m0
    def Anh(self,T,V):
        """
        Intrinsic anharmonicity correction to the Debye temperature.

        :param float T: Temperature.
        :param float V: Volume.
        """
        self.an_val=self.an(V)
        self.Anh_val = np.exp(1/2*self.an_val*T)
        return self.Anh_val
    def dAnhdT_V(self):
        self.dAnhdT_V_val = (1/2)*self.an_val*self.Anh_val
        return self.dAnhdT_V_val
    def d2AnhdT2_V(self):
        self.d2AnhdT2_V_val = (1/2)*self.an_val*self.dAnhdT_V_val
        return self.d2AnhdT2_V_val
    def d3AnhdT3_V(self):
        self.d3AnhdT3_V_val = (1/2)*self.an_val*self.d2AnhdT2_V_val
        return self.d3AnhdT3_V_val
    def d4AnhdT4_V(self):
        self.d4AnhdT4_V_val = (1/2)*self.an_val*self.d3AnhdT3_V_val
        return self.d4AnhdT4_V_val

    def dAnhdV_T(self,T,V):
        self.dAnhdV_T_val = self.m0*T*self.an_val*self.Anh_val/(2*V)
        return self.dAnhdV_T_val
    def d2AnhdV2_T(self,T,V):
        self.d2AnhdV2_T_val = self.dAnhdV_T_val*(T*self.an_val*self.m0+2*self.m0-2)/(2*V)
        return self.d2AnhdV2_T_val
    def d3AnhdV3_T(self,T,V):
        self.d3AnhdV3_T_val= self.dAnhdV_T_val*(T**2*self.an_val**2*self.m0**2+6*T*self.an_val*self.m0**2-6*T*self.an_val*self.m0+4*self.m0**2-12*self.m0+8)/(4*V**2)
        return self.d3AnhdV3_T_val

    def d4AnhdV4_T(self,T,V):
        self.d4AnhdT4_V_val = self.dAnhdV_T_val*(T**3*self.an_val**3*self.m0**3+12*T**2*self.an_val**2*self.m0**3-12*T**2*self.an_val**2*self.m0**2+28*T*self.an_val*self.m0**3-72*T*self.an_val*self.m0**2+44*T*self.an_val*self.m0+8*self.m0**3-48*self.m0**2+88*self.m0-48)/(8*V**3)
        return self.d4AnhdT4_V_val
    def d2AnhdVdT(self,T):
        self.d2AnhdVdT_val = self.dAnhdV_T_val*(1/T+self.an_val/2)
        return self.d2AnhdVdT_val
    def d3AnhdVdT2(self,T):
        self.d3AnhdVdT2_val = self.dAnhdV_T_val*self.an_val*(1/T+self.an_val/4)
        return self.d3AnhdVdT2_val
    def d3AnhdV2dT(self,T,V):
        self.d3AnhdV2dT_val = self.dAnhdV_T_val/V*(self.m0/T - 1/T + 3/2*self.an_val*self.m0 - self.an_val/2 + self.an_val**2*self.m0*T/4)
        return self.d3AnhdV2dT_val
